#!/bin/csh -f

if ( $#argv != 5 ) then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Output Image Name) (Sky Image1) (Sky Image2)"
    exit 1
endif



set FTABLE=$1
set NPOINT=$2
set HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
set SKY1=$4
set SKY2=$5
set OUTIM=$HEADER".fits"

echo "### Preparation ###"
set DITH=20
@ DITH2 = $DITH * 2
set NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME
gawk '{n=split($1,a,"/");print("ldata/"a[n])}' $FTABLE | sed 's/.fits/.imh/g' > ldata.tbl
sed 's/ldata/mldata/g' ldata.tbl > mldata.tbl
sed 's/.imh/.imh[257:768,257:768]/g' ldata.tbl > pldata.tbl
gawk -v nf=$NFRAME '{if(i<1) print($0); i++; if(i==nf) i=0;}' ldata.tbl > ldata.1.tbl
gawk -v nf=$NFRAME '{if(i>=1) print($0); i++; if(i==nf) i=0;}' ldata.tbl > ldata.2.tbl
sed 's/.imh/.imh[257:768,257:768]/g' ldata.1.tbl > pldata.1.tbl

echo "### Flat-fielding ###"
echo "imdel @ldata.tbl" > cl.com
echo "imarith @"$FTABLE" / ./flat.fits @ldata.tbl" >> cl.com
echo "logout" >> cl.com
cat cl.com | cl

echo "### Offset rejection in Quad 2 ###"
echo "offsetrej @ldata.tbl" | cl

echo "### Preparation for MKSUBGR command ###"
echo "del level.tbl" > cl.com
echo "imstatistics @pldata.1.tbl field=mean fo- > level.tbl" >> cl.com
echo "logout" >> cl.com
cat cl.com | cl
awk '{s+=$1;i++;}END{print(s/i)}' level.tbl > tmp.txt
echo "imdel sky1,sky2" > cl.com
echo "imdel sky*.imh" >> cl.com
echo "del bkg1.tbl,bkg2.tbl" >> cl.com
echo "logout" >> cl.com
cat cl.com | cl

echo "### MKSUBGR command ###"
gawk -v nf=1 '{print("mksubgr @ldata.1.tbl sky1 1 "nf" "$1" 257 768 257 768 bkg1.tbl")}' tmp.txt | cl
gawk -v nf=$NFRAME '{print("mksubgr @ldata.2.tbl sky2 1 "nf-1" "$1" 257 768 257 768 bkg2.tbl")}' tmp.txt | cl

echo "### Sky-subtraction ###"
echo "imarith @ldata.1.tbl - $SKY1 @ldata.1.tbl" > cl.com
echo "imarith @ldata.2.tbl - $SKY2 @ldata.2.tbl" >> cl.com
#echo "imdel tmp1*.imh,tmp1*.fits" >> cl.com
echo "imdel tmp2*.imh,tmp2*.fits" >> cl.com
echo "logout" >> cl.com
cat cl.com | cl

#ls sky1.???.imh > tmp.tbl
#sed -e 's/sky/tmp/g' tmp.tbl > tmp1.tbl
#echo "imarith @tmp.tbl - $SKY1 @tmp1.tbl" | cl

ls sky2.???.imh > tmp.tbl
sed -e 's/sky/tmp/g' tmp.tbl > tmp2.tbl
echo "imarith @tmp.tbl - $SKY2 @tmp2.tbl" | cl

echo "### Offset rejection in Quad 2 ###"
echo "offsetrej @tmp2.tbl" | cl

cat tmp2.tbl > tmpall.tbl
#cat tmp1.tbl tmp2.tbl > tmpall.tbl

echo "### Pattern Matching ###"
set PIXSCALE=0.111 
\rm tmphead.log 
echo "imhead @"$FTABLE" longheader=yes > tmphead.log" | cl 
grep "^SLT-PA" tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' > tmppa.log 
grep "^RA   " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{print($1*3600+$2*60+$3)}' > tmpra.log 
#grep "^DEC  " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{print(-1*($1*3600+$2*60+$3))}' > tmpdec.log 
grep "^DEC  " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{if(gsub("-","-",$1)>0)s=-1;else s=1;print(-1*s*(s*$1*3600+$2*60+$3))}' > tmpdec.log 
set RA=`gawk '{s+=$1}END{print(s/NR)}' tmpra.log` 
set DEC=`gawk '{s+=$1}END{print(s/NR)}' tmpdec.log` 
set PA=`gawk '{s+=$1}END{print(s/NR)}' tmppa.log` 
gawk -v ps=$PIXSCALE -v ra=$RA -v dec=$DEC '{print(($1-ra)*15*cos(dec/3600/180*3.14159)/ps)}' tmpra.log > tmpra2.log 
gawk -v ps=$PIXSCALE -v ra=$RA -v dec=$DEC '{print(($1-dec)/ps)}' tmpdec.log > tmpdec2.log 
paste tmpra2.log tmpdec2.log | gawk -v pa=$PA '{print(-1*(cos(pa)*$1+sin(pa)*$2),-1*(-sin(pa)*$1+cos(pa)*$2))}' > tmpoffset.log 
gawk -v nf=$NFRAME '{s1+=$1;s2+=$2;if(NR%nf==0){printf("%d %d\n",s1/nf,s2/nf);s1=0;s2=0}}' tmpoffset.log | gawk '{if(NR==1){dx=$1;dy=$2}print($1-dx,$2-dy)}'> tmp.offset1

echo "del tmp.sigma" > cl.com 
echo "imstatrej tmp2.001[300:600,300:600] 3 > tmp.sigma" >> cl.com
cat cl.com | cl 
set SIGMA=`gawk '{if(NR==1)print($3*2)}' tmp.sigma` 

echo "noao" > cl.com 
echo "digiphot" >> cl.com 
echo "apphot" >> cl.com
foreach filename (`cat tmp2.tbl`) 
  set coordfile=`echo $filename | sed 's/imh/coo/g'` 
  \rm $coordfile 
  echo "imdel tmpmed" >> cl.com
  echo "median "$filename" tmpmed 3 3" >> cl.com
  echo "daofind tmpmed output="$coordfile" fwhmpsf=15 thresho=4 verify- sigma="$SIGMA" sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> cl.com 
end 
echo "logout" >> cl.com
cat cl.com | cl  

# Pattern Matching 1st loop
@ i = 1 
foreach filename (`cat tmp2.tbl`)  
  set coordfile=`echo $filename | sed 's/imh/coo/g'`  
  set coordfile2=`echo $filename | sed 's/.imh/2.coo/g'`  
  set XOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($1)}'`
  set YOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($2)}'`
  grep -v "^#" $coordfile | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $coordfile2
  @ i = $i + 1
end

# Fine Tuning of Offset
@ i = 1 
foreach filename (`cat tmp2.tbl`)  
  set coordfile2=`echo $filename | sed 's/.imh/2.coo/g'`  
  echo $coordfile2 $i
  if ( $i == 1 ) then
    set basefile=$coordfile2
    echo "0 0" > tmp.offset2
  else
    ./matchpat $basefile $coordfile2 20 >> tmp.offset2
  endif
  @ i = $i + 1
end

mv tmp.offset1 tmp.offsettmp

paste tmp.offsettmp tmp.offset2 | gawk '{print($1+$3,$2+$4)}' > tmp.offset1

# Pattern Matching 2nd loop
@ i = 1 
foreach filename (`cat tmp2.tbl`)  
  set coordfile=`echo $filename | sed 's/imh/coo/g'`  
  set coordfile3=`echo $filename | sed 's/.imh/3.coo/g'`  
  set XOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($1)}'`
  set YOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($2)}'`
  grep -v "^#" $coordfile | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $coordfile3
  @ i = $i + 1
end

# Fine Tuning of Offset
@ i = 1 
foreach filename (`cat tmp2.tbl`)  
  set coordfile3=`echo $filename | sed 's/.imh/3.coo/g'`  
  if ( $i == 1 ) then
    set basefile=$coordfile3
    echo "0 0" > tmp.offset2
  else
    ./matchpat $basefile $coordfile3 5 >> tmp.offset2
  endif
  @ i = $i + 1
end

paste tmp.offset1 tmp.offset2 | gawk '{print($1+$3,$2+$4)}' > tmp.offset
#cat tmp.offset tmp.offset > tmp.offset

echo "### Shift and Combine ###"
echo "imdel tmp.tot" > cl.com
echo "imcombine @tmpall.tbl tmp.tot combine=median reject=sigclip outtype='real' lsigma=2 hsigma=2 offset=tmp.offset" >> cl.com
echo "logout" >> cl.com
cat cl.com | cl
\rm offset.$HEADER
cp tmp.offset offset.$HEADER
\rm RESULTS/offset.$HEADER
mv offset.$HEADER RESULTS/

echo "### Object Detection ###"
#calculate background sigma
echo "del tmp.sigma" > cl.com
echo "imstatrej tmp.tot[300:600,300:600] 3 > tmp.sigma" >> cl.com
echo "logout" >> cl.com
cat cl.com | cl

set SIGMA=`gawk '{if(NR==1)print($3*2)}' tmp.sigma`
echo "noao" > cl.com
echo "digiphot" >> cl.com
echo "apphot" >> cl.com
echo "del tmp.tot.coo" >> cl.com
echo "imdel tmp1.fits,tmp1.imh" >> cl.com
echo "gauss tmp.tot tmp1 2.5" >> cl.com
if ($DITH > 0) then
  echo "imshift tmp1 tmp1 "$DITH" "$DITH" boundary=constant">> cl.com
  echo "imshift tmp1 tmp1 -"$DITH2" 0 boundary=constant" >> cl.com
  echo "imshift tmp1 tmp1 0 -"$DITH2" boundary=constant" >> cl.com
  echo "imshift tmp1 tmp1 "$DITH" "$DITH" boundary=constant" >> cl.com
endif
echo "daofind tmp1 output="tmp.tot.coo" fwhmpsf=15 thresho=4 verify- sigma="$SIGMA "sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> cl.com
echo "displ tmp.tot 1 fi+" >> cl.com
#echo "tvmark 1 tmp.tot.coo number+" >> cl.com
echo "imdel RESULTS/$OUTIM" >> cl.com
echo "imdel RESULTS/$HEADER.sky1.fits" >> cl.com
echo "imdel RESULTS/$HEADER.sky2.fits" >> cl.com
echo "imcopy tmp.tot RESULTS/$OUTIM" >> cl.com
echo "imcopy sky1.imh RESULTS/$HEADER.sky1.fits" >> cl.com
echo "imcopy sky2.imh RESULTS/$HEADER.sky2.fits" >> cl.com
echo "logout" >> cl.com
cat cl.com | cl
./offsettbl tmp.tot.coo RESULTS/offset.$HEADER MASK/$HEADER

echo "### $0 Finished ###"
echo $NPOINT" points dithering."
echo "Output Image is "RESULTS/$OUTIM
echo "Sky Image is "RESULTS/$HEADER.sky1.fits
echo "             "RESULTS/$HEADER.sky2.fits
echo "Offset pattern file is saved as "RESULTS/offset.$HEADER
echo "Offset pattern is : "
cat RESULTS/offset.$HEADER
echo "Sigma value for daofind is " $SIGMA
echo "Mask of objcts are saved as"
ls MASK/$HEADER.0??.tbl | cat
echo "."
